/**
 * DeleteIntersectionBorders_8C
 * Created on 29th January 2008, 13:01
 *
 * This plug-in eliminates the borders pixels between the different particles in
 * the stack.
 *
 * Copyright (c) 2008 by Manuel Guillermo Forero-Vargas
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

public class DeleteIntersectionBorders_8C implements PlugInFilter
{
    protected   ImagePlus imp;
    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
	{
            showAbout();
            return DONE;
	}
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
        this.imp=imp;
        return IJ.setupDialog(imp,DOES_16);
    }

    public void run(ImageProcessor ip)
    {
        int	a,i,j,x,X=0,y,Y=0;
	int	height=ip.getHeight();
	int	width=ip.getWidth();
	short[][]   pixels=new short[width][height];

        //----------------------------------------------------------------------
        //Does a copy of the original stack 
        short[]  pixelsO=(short[])((ShortProcessor)ip).getPixels();
        for(y=0;y<height;y++)
        {
            a=y*width;
            for(x=0;x<width;x++)
                pixels[x][y]=pixelsO[a+x];
        }
        //----------------------------------------------------------------------
            for(y=1;y<height-1;y++)
                for(x=1;x<width-1;x++)
                    if (pixels[x][y]!=0)
                    {
                        a=pixels[x][y];
                        //--------------------------------------------------------------
                        // Revision of neighbours
                        for (i=0;i<8;i++)
                        {
                            switch (i)
                            {
                                case 0://(x+1,y)
                                    X=x+1;
                                    Y=y;
                                    break;
                                case 1://(x+1,y-1)
                                    Y--;
                                    break;
                                case 2://(x,y-1)
                                    X--;
                                    break;
                                case 3://(x-1,y-1)
                                    X--;
                                    break;
                                case 4://(x-1,y)
                                    Y++;
                                    break;
                                case 5://(x-1,y+1)
                                    Y++;
                                    break;
                                case 6://(x,y+1)
                                    X++;
                                    break;
                                case 7://(x+1,y+1)
                                    X++;
                                    break;
                            }
                            if(pixels[X][Y]>=0&&pixels[X][Y]!=a)
                                pixels[X][Y]=0;
                        }
                    }
            //------------------------------------------------------------------
            //Charge the result
            for(y=0;y<height;y++)
            {
                a=y*width;
                for(x=0;x<width;x++)
                    pixelsO[a+x]=pixels[x][y];
            }
    }

    void showAbout()
    {
        IJ.showMessage("About DeleteIntersectionBorders_8C...",
        "This plug-in eliminates the borders pixels between the different particles in the image.");
    }
}
