/**
* Erosion3D_.java
* Created on January 2005 by Manuel Guillermo Forero-Vargas
* e-mail: , mgforero@yahoo.es
*
* Erosion 3D algorith by using 6-connectivity
* Function: This plugin erodes white objects (white=255)
*
* This plugin do not create a new image
*
* Copyright (c) 2005 by Manuel Guillermo Forero-Vargas
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;

public class Erosion3D_ implements PlugInFilter
 {
    protected   ImagePlus   imp;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        //Verification of binary imp. Taken from Landini
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
        ImageStatistics stats=imp.getStatistics();
        if (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount)
        {
            IJ.error("8-bit binary imp (0 and 255) required.");
            return DONE;
        }
        this.imp=imp;
	return DOES_8G+DOES_8C+STACK_REQUIRED+NO_UNDO;
    }

    public void run(ImageProcessor ip)
    {
        ImageProcessor  ipstack;
        final ImageStack    stack=imp.getStack();
        final int   height=ip.getHeight();
        final int   width=ip.getWidth();
        final int   depth=stack.getSize();//Number of images
        int     x,y,z;
	int[][][]   pixels=new int[depth][width+2][height+2];

        //----------------------------------------------------------------------
        //A copy of the original imp is done
        //----------------------------------------------------------------------
        for (z=0;z<depth;z++)
        {
            ipstack=stack.getProcessor(z+1);
            for(y=0;y<height;y++)
                for(x=0;x<width;x++)
                    pixels[z][x+1][y+1]=ipstack.getPixel(x,y);//original
        }
        //----------------------------------------------------------------------
        //The erosion is applied
        for (z=0;z<depth;z++)
        {
            ipstack=stack.getProcessor(z+1);            
            if (z==0)//Erosion of first image in the stack
            {
                for (y=1;y<=height;y++)
                    for(x=1;x<=width;x++)
                        if(pixels[0][x][y]>0&(pixels[0][x-1][y-1]+pixels[0][x][y-1]+pixels[0][x+1][y-1]+
                        pixels[0][x-1][y]+pixels[0][x+1][y]+pixels[0][x-1][y+1]+
                        pixels[0][x][y+1]+pixels[0][x+1][y+1]+
                        pixels[1][x-1][y-1]+pixels[1][x][y-1]+pixels[1][x+1][y-1]+
                        pixels[1][x-1][y]+pixels[1][x][y]+pixels[1][x+1][y]+
                        pixels[1][x-1][y+1]+pixels[1][x][y+1]+pixels[1][x+1][y+1])<4335)//17 pixels*255=4335
                            ipstack.putPixel(x-1,y-1,0);
            }
            else
            {
                if (z==depth-1)//Erosion of last image in the stack
                {
                    for (y=1;y<=height;y++)
                        for(x=1;x<=width;x++)
                            if(pixels[z][x][y]>0&(pixels[z-1][x-1][y-1]+pixels[z-1][x][y-1]+pixels[z-1][x+1][y-1]+
                            pixels[z-1][x-1][y]+pixels[z-1][x][y]+pixels[z-1][x+1][y]+
                            pixels[z-1][x-1][y+1]+pixels[z-1][x][y+1]+pixels[z-1][x+1][y+1]+
                            pixels[z][x-1][y-1]+pixels[z][x][y-1]+pixels[z][x+1][y-1]+
                            pixels[z][x-1][y]+pixels[z][x+1][y]+pixels[z][x-1][y+1]+
                            pixels[z][x][y+1]+pixels[z][x+1][y+1])<4335)
                                ipstack.putPixel(x-1,y-1,0);
                }
                else
                {
                    for (y=1;y<=height;y++)
                        for(x=1;x<=width;x++)
                            if(pixels[z][x][y]>0&(pixels[z-1][x-1][y-1]+pixels[z-1][x][y-1]+pixels[z-1][x+1][y-1]+
                            pixels[z-1][x-1][y]+pixels[z-1][x][y]+pixels[z-1][x+1][y]+
                            pixels[z-1][x-1][y+1]+pixels[z-1][x][y+1]+pixels[z-1][x+1][y+1]+
                            pixels[z][x-1][y-1]+pixels[z][x][y-1]+pixels[z][x+1][y-1]+
                            pixels[z][x-1][y]+pixels[z][x+1][y]+pixels[z][x-1][y+1]+
                            pixels[z][x][y+1]+pixels[z][x+1][y+1]+
                            pixels[z+1][x-1][y-1]+pixels[z+1][x][y-1]+pixels[z+1][x+1][y-1]+
                            pixels[z+1][x-1][y]+pixels[z+1][x][y]+pixels[z+1][x+1][y]+
                            pixels[z+1][x-1][y+1]+pixels[z+1][x][y+1]+pixels[z+1][x+1][y+1])<6630)//26 pixels*255=6630
                                ipstack.putPixel(x-1,y-1,0);
                }
            }
        }
        //----------------------------------------------------------------------
    imp.show();
    imp.updateAndDraw();
    }

    void showAbout()
    {
        IJ.showMessage("About Erosion3D_...",
        "This plug-in filter erodes white pixels.");
    }
}
