/**
 * Fast_dome.java
 * Created on 1 march 2007, 15:15
 *
 * Fast dome algorithm.
 * Function: Localises the domes in an image by using the Vincent's secuential
 * method.
 *
 * This plugin do not create a new image.
 *
 * Copyright (c) 2007 by Manuel Guillermo Forero-Vargas
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import ij.*;
import ij.gui.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;

public class Fast_dome implements PlugInFilter
{
    protected  ImagePlus imp;
    protected static int domeheight=20;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        GenericDialog gd=new GenericDialog("Dome");
        gd.addNumericField ("Height",domeheight,0);
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        domeheight=(int)gd.getNextNumber();
        this.imp=imp;
        return IJ.setupDialog(imp,DOES_8G+DOES_8C);//Works in stacks or single
        //grey level images
    }

    public void run(ImageProcessor ip)
    {
        boolean change=true;//Tells if the reconstructed image has changed.
        final int height=ip.getHeight();
        final int width=ip.getWidth();
        final int size=height*width;
        int a,b,c,i,max,q,x,X=0,y,Y=0,z;
        int[] originalPixels=new int[size];//Original image
        int[] darkPixels=new int[size];//Dark image
        byte[] resultPixels=(byte[])ip.getPixels();

        for(i=0;i<size;i++)
        {
            originalPixels[i]=resultPixels[i]&0xff;
            darkPixels[i]=originalPixels[i]-domeheight;//darkened
        }
        while (change)
        {
            change=false;
            //------------------------------------------------------------------
            //Reconstruccion of the image
            for (y=0;y<height;y++)
            {
                a=y*width;
                for (x=0;x<width;x++)
                {
                    b=a+x;
                    for (max=0,i=0;i<5;i++)
                    {
                        switch (i)
                        {
                            case 0://(x+1,y-1)
                                X=x+1;
                                Y=y-1;
                                break;
                            case 1://(x,y-1)
                                X--;
                                break;
                            case 2://(x-1,y-1)
                                X--;
                                break;
                            case 3://(x-1,y)
                                Y++;
                                break;
                            case 4://(x,y)
                                X++;
                                break;
                        }
                        if(X>=0&&X<width&&Y>=0&&Y<height&&(q=darkPixels[Y*width+X])>max)
                            max=q;
                    }
                    c=darkPixels[b];
                    darkPixels[b]=max<originalPixels[b]?max:originalPixels[b];
                    if (c!=darkPixels[b])
                        change=true;
                }
            }
            for (y=height-1;y>=0;y--)
            {  
                a=y*width;
                for (x=width-1;x>=0;x--)
                {
                    b=a+x;
                    for (max=0,i=0;i<5;i++)
                    {
                        switch (i)
                        {
                            case 0://(x,y)
                                X=x;
                                Y=y;
                                break;
                            case 1://(x+1,y)
                                X++;
                                break;
                            case 2://(x+1,y+1)
                                Y++;
                                break;
                            case 3://(x,y+1)
                                X--;
                                break;
                            case 4://(x-1,y+1)
                                X--;
                                break;
                        }
                        if(X>=0&&X<width&&Y>=0&&Y<height&&(q=darkPixels[Y*width+X])>max)
                            max=q;
                    }
                    c=darkPixels[b];
                    darkPixels[b]=max<originalPixels[b]?max:originalPixels[b];
                    if (c!=darkPixels[b])
                        change=true;
                }
            }
        }
        //----------------------------------------------------------------------
        //Gets the dome
        for(i=0;i<size;i++)
            resultPixels[i]=(byte)(originalPixels[i]-darkPixels[i]);
        imp.show();
        imp.updateAndDraw();
    }

    void showAbout()
    {
        IJ.showMessage("About Fast_dome...",
        "This plug-in determines the domes of 8-bit images.");
    }
}
