/*
* Fast_hybrid_grey_scale_reconstruction.java
* Created on 1 march 2007, 14:41
*
* Fast hybrid grey scale reconstruction algorithm.
* This plugin do a morphologic reconstruction by using the Vincent's hybrid
* method.
*
* This plugin do not create a new image. The result in charged in the seed image.
* If the seed image is not labelled, it is assumed just one seed by object, so
* or more seeds cannot be assigned to the same region. In this case, you must be
* sure of having just one pixel by seed.
*
* Copyright (c) 2007 by Manuel Guillermo Forero-Vargas
* e-mail: mgforero@yahoo.es
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import ij.*;
import ij.gui.*;
import ij.plugin.*;
import java.awt.*;//Employed to use the class Point
import java.util.*;//Employed to use the LinkedList class

public class Fast_hybrid_grey_scale_reconstruction implements PlugIn
{
    public void run(String arg)
    {
        int a,b,c,i,max,offset,q,x,X=0,y,Y=0,z;
        
        if (arg.equals("about"))
        {
            showAbout();
            return;
        }
        //----------------------------------------------------------------------
        String[] titles;
        int[] wList=WindowManager.getIDList();
        if (wList==null||wList.length<2)
        {
            IJ.showMessage("Grey scale reconstruction","There must be at least two windows open");
            return;
        }
        titles=new String[wList.length];
        for (i=0;i<wList.length;i++)
        {
            ImagePlus imp=WindowManager.getImage(wList[i]);
            titles[i]=imp!=null?imp.getTitle():"";
        }
        //----------------------------------------------------------------------
	int IndexMask,IndexSeed;
        GenericDialog gd=new GenericDialog("Grey scale reconstruction");
        gd.addChoice("Seed:",titles,titles[0]);
        gd.addChoice("Mask:",titles,titles[1]);
        gd.showDialog();
        if (gd.wasCanceled())
            return;
        IndexSeed=gd.getNextChoiceIndex();
        IndexMask=gd.getNextChoiceIndex();
        //----------------------------------------------------------------------
        ImagePlus impSeed=WindowManager.getImage(wList[IndexSeed]);
        ImagePlus impMask=WindowManager.getImage(wList[IndexMask]);
        if (impSeed.getBitDepth()!=8||impMask.getBitDepth()!=8)
        {
            IJ.error("8-bit images required");
            return;
        }
        final int depth=impSeed.getStackSize();
        if (depth!=impMask.getStackSize())
        {
            IJ.error("The number of images of both stacks must be the same");
            return;
        }
        //----------------------------------------------------------------------
	// 0. Inicializar los datos
        //----------------------------------------------------------------------
        final int height=impSeed.getHeight();
        final int width=impSeed.getWidth();
        if ((height!=impMask.getHeight())||(width!=impMask.getWidth()))
        {
            IJ.error("The size of the images of both stacks must be the same");
            return;
        }
        final int size=height*width;
        byte[]  maskPixels;
        byte[]  seedPixels;
        ImageStack stackMask=impMask.getStack();
        ImageStack stackSeed=impSeed.getStack();
        int[] darkPixels=new int[size];//Dark pixels
        int[] originalPixels=new int[size];//Dark pixels
        LinkedList fifo=new LinkedList();
        Point p;
        for (z=1;z<=depth;z++)
        {
            IJ.showStatus("Iteration " +z +"/" + depth + "...");
            IJ.showProgress(z,depth);
            maskPixels=(byte[])stackMask.getProcessor(z).getPixels();
            seedPixels=(byte[])stackSeed.getProcessor(z).getPixels();
            for(i=0;i<size;i++)
            {
                originalPixels[i]=maskPixels[i]&0xff;
                darkPixels[i]=seedPixels[i]&0xff;
            }
            //------------------------------------------------------------------
            //Reconstruccion de la imagen
            for (y=0;y<height;y++)
            {
                a=y*width;
                for (x=0;x<width;x++)//Se obtienen los pixeles de la ventana correspondiente
                {
                    b=a+x;
                    for (max=0,i=0;i<5;i++)
                    {
                        switch (i)
                        {
                            case 0://(x+1,y-1)
                                X=x+1;
                                Y=y-1;
                                break;
                            case 1://(x,y-1)
                                X--;
                                break;
                            case 2://(x-1,y-1)
                                X--;
                                break;
                            case 3://(x-1,y)
                                Y++;
                                break;
                            case 4://(x,y)
                                X++;
                                break;
                        }
                        if(X>=0&&X<width&&Y>=0&&Y<height&&(q=darkPixels[Y*width+X])>max)
                            max=q;
                    }
                    darkPixels[b]=max<originalPixels[b]?max:originalPixels[b];
                }
            }
            for (y=height-1;y>=0;y--)
            {
                a=y*width;
                for (x=width-1;x>=0;x--)//Se obtienen los pixeles de la ventana correspondiente
                {
                    b=a+x;
                    for (max=0,i=0;i<5;i++)
                    {
                        switch (i)
                        {
                            case 0://(x,y)
                                X=x;
                                Y=y;
                                break;
                            case 1://(x+1,y)
                                X++;
                                break;
                            case 2://(x+1,y+1)
                                Y++;
                                break;
                            case 3://(x,y+1)
                                X--;
                                break;
                            case 4://(x-1,y+1)
                                X--;
                                break;
                        }
                        if(X>=0&&X<width&&Y>=0&&Y<height&&(q=darkPixels[Y*width+X])>max)
                            max=q;
                    }
                    darkPixels[b]=max<originalPixels[b]?max:originalPixels[b];
                    for (i=1;i<5;i++)
                    {
                        switch (i)
                        {
                            case 1://(x+1,y)
                                X=x+1;
                                Y=y;
                                break;
                            case 2://(x+1,y+1)
                                Y++;
                                break;
                            case 3://(x,y+1)
                                X--;
                                break;
                            case 4://(x-1,y+1)
                                X--;
                                break;
                        }
                        if(X>=0&&X<width&&Y>=0&&Y<height)
                        {
                            c=Y*width+X;
                            q=darkPixels[c];
                            if(q<darkPixels[b]&&q<originalPixels[c])
                                fifo.addLast(new Point(x,y));
                        }
                    }
                }
            }
            while (!fifo.isEmpty())
            {
                //p=(Point)fifo.pollFirst();//JAVA 1.6
                p=(Point)fifo.removeFirst();//JAVA 1.4
                x=(int)p.getX();//x ordinate
                y=(int)p.getY();//y ordinate
                a=y*width+x;
                for (Y=y-1;Y<=y+1;Y++)//Barre los pixeles dentro de la subventana
                    if (Y>=0&&Y<height)
                    {
                        offset=Y*width;
                        for (X=x-1;X<=x+1;X++)
                            if (X>=0&&X<width)
                            {
                                c=offset+X;
                                if (darkPixels[c]<darkPixels[a]&&darkPixels[c]!=originalPixels[c])
                                {
                                    darkPixels[c]=darkPixels[a]<originalPixels[c]?darkPixels[a]:originalPixels[c];
                                    fifo.addLast(new Point(X,Y));
                                }
                            }
                    }
            }
            //----------------------------------------------------------------------
            //Update of the result
            for(i=0;i<size;i++)
                seedPixels[i]=(byte)darkPixels[i];
        }
        //----------------------------------------------------------------------
        impSeed.show();
        impSeed.updateAndDraw();
     }

    void showAbout()
    {
        IJ.showMessage("About Fast hybrid grey scale reconstruction...",
        "This plug-in makes the grey scale reconstruction of 8-bit images.");
    }
}
