/**
 * Kapur_.java
 * Created on 26 june 2005, 19:23
 *
 * Kapur thresholding algorithm
 * This plugin do not create a new image, if you want create a new image use RATSS_
 *
 * Copyright (c) 2005 by Manuel Guillermo Forero-Vargas
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.measure.*;

public class Kapur_ implements PlugInFilter, Measurements
{
    protected   ImagePlus   imp;
    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        this.imp=imp;
        return IJ.setupDialog(imp,DOES_8G+DOES_8C);
    }

    public void run(ImageProcessor ip)
    {
        double[] normalizedHistogram;
        double Pt1=0;//Probability of the class 1
        double Pt2=0;//Probability of the class2
	double H1,H2;//Entropy of the classes 1 and 2
	double HOpt=0;
	double H;
        ImageStatistics stats;
	int t;
	int threshold;
	int i;
	int minq,maxq;//Limits values of the histogram
        int size=ip.getHeight()*ip.getWidth();
        int[] histogram=((ByteProcessor) ip).getHistogram();

        normalizedHistogram=new double[256];   
        //----------------------------------------------------------------------
        //Gets limits of the histogram
        //Returns the first intensity level that appears in the histogram
        minq=0;
        while (histogram[minq]==0)//Sweeps the histogram
            minq++;
        //Returns the last intensity level that appears in the histogram
        maxq=255;//Levels-1;
        while (histogram[maxq]==0)
            maxq--;
        //--------------------------------------------------------------
        for(i=minq;i<=maxq;i++)
            normalizedHistogram[i]=(double)histogram[i]/(double)size;
	for (t=threshold=minq;t<maxq;t++)
	{
            Pt1+=normalizedHistogram[t];
            Pt2=1.0-Pt1;
            //Calculates Hn and Hb		
            for (H1=i=0;i<=t;i++)
                if (normalizedHistogram[i]>0)
                    H1-=(normalizedHistogram[i]/Pt1)*Math.log(normalizedHistogram[i]/Pt1);
            for (H2=0,i=t+1;i<=maxq;i++)
                if (normalizedHistogram[i]>0)
                    H2-=(normalizedHistogram[i]/Pt2)*Math.log(normalizedHistogram[i]/Pt2);
            H=H1+H2;
            if (H>HOpt)
            {
                HOpt=H;
                threshold=t;
            }					
	}
        ip.threshold(threshold);
    }

    void showAbout()
    {
        IJ.showMessage("About Kapur_...",
        "This plug-in calculates the Kapur threshold of 8-bit images.");
    }
}
