/**
 * Kmeans_thresholding.java
 * Created on 9 December 2008, 17:03
 *
 * K means thresholding algorithm
 * This plugin do not create a new image
 *
 * Copyright (c) 2008 by Manuel Guillermo Forero-Vargas
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import ij.*;
import ij.process.*;
import ij.gui.*;
import ij.plugin.filter.*;

public class Kmeans_thresholding implements PlugInFilter
{
	ImagePlus imp;
    static int clusters=2;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        this.imp=imp;
      	//----------------------------------------------------------------------
        /*Creates a dialog box, allowing the user to enter the requested
        number of clusters.*/
        GenericDialog gd=new GenericDialog("Kmeans_thresholding",IJ.getInstance());
        gd.addNumericField("Clusters:",clusters,0,2,"");
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        clusters=(int)gd.getNextNumber();
        return IJ.setupDialog(imp,DOES_8G+DOES_8C);
    }

	public void run(ImageProcessor ip)
    {
        byte[]	binarypixels=(byte[])ip.getPixels();
        int i,k;
        float ftolerancia=0.01f;
        float error;
        int[] clase=new int[256];//Keeps the class to which belongs each greylevel
        float[] m=new float[clusters];
        float[] mOld=new float[clusters];
        int[] histogram=((ByteProcessor) ip).getHistogram();
        int[] sumo=new int[clusters];
        int[] sumPixelso=new int[clusters];

        float minDist,dist;
        int minGrey,maxGrey;//Limits of the histogram
        int size=ip.getHeight()*ip.getWidth();
        //--------------------------------------------------------------
        //Gets limits of the histogram
        //Returns the first intensity level that appears in the histogram
        minGrey=0;
        while (histogram[minGrey]==0)
            minGrey++;
        //Returns the last intensity level that appears in the histogram
        maxGrey=255;//Levels-1;
        while (histogram[maxGrey]==0)
            maxGrey--;
        //----------------------------------------------------------------------
        // Initialisation
        for (k=0,i=1;i<clusters*2;k++,i+=2)//Find the initial values of the clusters centres
            m[k]=(maxGrey-minGrey)*i/(clusters*2)+minGrey;
        //----------------------------------------------------------------------
        error=10000000.0f;
        while (error>ftolerancia)
        {
            error=-1.0f;
            //------------------------------------------------------------------
            // Greylevels are asigned to their clasess
            for (i=minGrey;i<=maxGrey;i++)
                for (minDist=500f,k=0;k<clusters;k++)//Find the closest centre
                {
                    dist=Math.abs((float)i-m[k]);
                    if (dist<minDist)
                    {
                        minDist=dist;
                        clase[i]=k;
                    }
                }
            //------------------------------------------------------------------
            for (k=0;k<clusters;k++)
            {
                sumo[k]=sumPixelso[k]=0;
                mOld[k]=m[k];
            }
            for (i=minGrey;i<=maxGrey;i++)
            {
                sumo[clase[i]]+=histogram[i];
                sumPixelso[clase[i]]+=i*histogram[i];
            }
            for (k=0;k<clusters;k++)
            {
                m[k]=sumPixelso[k]/sumo[k];
                 if (Math.abs(m[k]-mOld[k])>error)
                    error=Math.abs(m[k]-mOld[k]);
            }
        }
        //Loads result
        //----------------------------------------------------------
        int a=255/(clusters-1);
        for (i=0;i<size;i++)
            binarypixels[i]=(byte)(clase[binarypixels[i]&0xff]*a+0.5);
        imp.show();
        imp.updateAndDraw();
	}

    void showAbout()
    {
        IJ.showMessage("About Kmeans_thresholding...",
        "This plug-in filter calculates the Kmeans clusters of 8-bit images.");
    }
}
