/**
 * LocalKapur_.java
 * Created on 22 october 2005, 19:41
 *
 * Local Kapur thresholding algorithm
// Funcin para umbralizacin local de la imagen segn el tamao de ventana dado.
// El tamao de la ventana debe ser impar o de lo contrario retorna un mensaje de error
// Por defecto se toma un ancho de ventana de 15 and the grey difference of 40
// Slo funciona para imagenes en niveles de gris
 * This plugin do not create a new image
 *
 * Copyright (c) 2005 by Manuel Guillermo Forero-Vargas
 * e-mail: m.g.forerovargas@bham.ac.uk
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import java.awt.*;
import ij.gui.*;

public class LocalKapur_ implements PlugInFilter
{
    protected   ImagePlus   imp;
    static int mingrey=1;//Minimum grey level of the pixels in a local window
    static int mingreydifference=1;//Minimum grey level difference among the pixels in a local window
    static int mint=0;//Minimum threshold in a local window
    static int windowwidth=15;//Width of the local windows

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        this.imp=imp;
        //----------------------------------------------------------------------
        /*Creates a dialog box, allowing the user to enter the requested
        windowwidth and minimum grey level difference by window.*/
        GenericDialog gd=new GenericDialog("Local Kapur",IJ.getInstance());
        gd.addNumericField("Window Width:",windowwidth,0,2,"pixels");
        gd.addNumericField("Minimum_grey_level_difference by window:",mingreydifference,0);
        gd.addNumericField("Minimum_grey_level:",mingrey,0);
        gd.addNumericField("Minimum_threshold:",mint,0);
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        windowwidth=(int)gd.getNextNumber();
	if (!(windowwidth%2==1))//Si el ancho de la ventana no es impar
        {
            IJ.showMessage("Error","Width of the window must be odd");
            return DONE;
        }
        mingreydifference=(int)gd.getNextNumber();
        mingrey=(int)gd.getNextNumber();
        mint=(int)gd.getNextNumber();
        return IJ.setupDialog(imp,DOES_8G);//Works in stacks or single grey level images
    }

    public void run(ImageProcessor ip)
    {
        byte[]	binarypixels=(byte[])ip.getPixels();
        double Pt1=0;
        double Pt2=0;
        double H1,H2;//Entropa de los objetos negro y blanco
        double fOpt=0;
        double f;
	int a,b,c,g,i,j,offset,q;
	int threshold;
	int minGris,maxGris;// Lmites del histograma
        int height=ip.getHeight();
	int x,y;//Para recorrer la subimagen
        int width=ip.getWidth();     
        int midwidth=windowwidth/2;//Mitad del ancho de la ventana
        int subsize=windowwidth*windowwidth;//Tamao de la subimagen
        int bigheight=height+2*midwidth;
        int bigwidth=width+2*midwidth;
	int[] pixels=new int[bigheight*bigwidth];

        //----------------------------------------------------------------------
        /*Se crea copia de mayor tamao de la imagen original donde los bordes
         se duplican mediante reflexion.*/
        //----------------------------------------------------------------------
        //Copia los pixeles del centro de la imagen
        for(i=0;i<height;i++)
        {
            a=(i+midwidth)*bigwidth;
            b=i*width;
            c=a+midwidth;
            for(j=0;j<width;j++)
                pixels[c+j]=binarypixels[b+j]&0xff;
        }
        //----------------------------------------------------------------------
        //Duplica los pixeles de las partes superior e inferior de la imagen
        for (j=midwidth;j<midwidth+width;j++)
        {
            a=2*midwidth-1;//Duplica parte superior
            for (i=0;i<midwidth;i++)
                pixels[i*bigwidth+j]=pixels[(a-i)*bigwidth+j];
            a=2*bigheight-2*midwidth-1;//Duplica parte inferior
            for (i=midwidth+height;i<bigheight;i++)
                pixels[i*bigwidth+j]=pixels[(a-i)*bigwidth+j];
        }
        //----------------------------------------------------------------------
        //Duplica los pixeles de las partes izquierda y derecha de la imagen
        for (i=0;i<bigheight;i++)
        {
            a=2*midwidth-1;//Duplica parte izquierda
            b=i*bigwidth;
            for (j=0;j<midwidth;j++)
                pixels[b+j]=pixels[b+(a-j)];
            a=2*bigwidth-2*midwidth-1;//Duplica parte derecha
            c=b+a;
            for (j=midwidth+width;j<bigwidth;j++)
                pixels[b+j]=pixels[c-j];
        }
//------------------------------------------------------------------------------
//Calculo del umbral local de cada ventana y binarizacion de cada pixel
        for (i=midwidth;i<bigheight-midwidth;i++)
        {
            a=i*bigwidth;
            b=(i-midwidth)*width;
            c=b-midwidth;
            for (j=midwidth;j<bigwidth-midwidth;j++)//Se obtienen los pixeles de la ventana correspondiente
            {
                if (pixels[a+j]<mingrey)
                    binarypixels[c+j]=0;
                else
                {
                    double[] normalizedHistogram=new double[256];//Se crea el histograma de la subventana inicializado en cero
                    for (y=i-midwidth;y<=i+midwidth;y++)//Barre los pixeles dentro de la subventana
                    {
                        offset=y*bigwidth;
                        for (x=j-midwidth;x<=j+midwidth;x++)
                            normalizedHistogram[pixels[offset+x]]++;//Clculo del histograma
                    }
                    //--------------------------------------------------------------
                    //Obtener los lmites del histograma
                    //Regresa el primer nivel de intensidad que aparece en el histograma
                    minGris=0;
                    while (normalizedHistogram[minGris]==0)//Barre el histograma
                        minGris++;//Cuando encuentra el primer valor!=0 lo regresa
                    //Regresa el ltimo nivel de intensidad que aparece en el histograma a pintar
                    maxGris=255;//Levels-1;
                    while (normalizedHistogram[maxGris]==0)
                        maxGris--;
                    //--------------------------------------------------------------
                    //Regresa el nivel 0 si la diferencia entre los niveles maximo y
                    //minimo es menor a mingreydifference
                    if (maxGris-minGris<mingreydifference)
                        binarypixels[c+j]=0;
                    //--------------------------------------------------------------
                    else
                    {
                        for(q=minGris;q<=maxGris;q++)//Obtiene histograma normalizado
                            normalizedHistogram[q]/=(double)subsize;        
                        //----------------------------------------------------------
                        //Calculus of the threshold by the Kapur method
                        Pt1=0;
                        Pt2=0;
                        fOpt=0;
                        for (q=threshold=minGris;q<maxGris;q++)
                        {
                            Pt1+=normalizedHistogram[q];
                            Pt2=1.0-Pt1;
                            // Calcular Hn y Hb		
                            for (H1=g=0;g<=q;g++)
                                if (normalizedHistogram[g]>0)
                                    H1-=(normalizedHistogram[g]/Pt1)*Math.log(normalizedHistogram[g]/Pt1);
                            for (H2=0,g=q+1;g<=maxGris;g++)
                                if (normalizedHistogram[g]>0)
                                    H2-=(normalizedHistogram[g]/Pt2)*Math.log(normalizedHistogram[g]/Pt2);
                            f=H1+H2;
                            if (f>fOpt)
                            {
                                fOpt=f;
                                threshold=q;
                            }					
                        }
                        //----------------------------------------------------------              
                        if (threshold<mint)
                            binarypixels[c+j]=0;
                        else
                            binarypixels[c+j]=pixels[a+j]<=threshold?(byte)0:(byte)255;
                    }
                }
            }
        }
    imp.show();
    imp.updateAndDraw();
//  IJ.showMessage("Threshold : "+threshold);
    }

    void showAbout()
    {
        IJ.showMessage("About LocalKapur_...",
        "This plug-in filter calculates the Local Kapur Threshold of 8-bit images.");
    }
}