/**
 * Majority black filter algorithm
 * This plugin do not create a new image
 *
 * Copyright (c) 2005 by Manuel Guillermo Forero-Vargas
 * e-mail: foreromg@bioex1.bham.ac.uk
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import java.awt.*;
import ij.gui.*;

public class MajorityBlack_ implements PlugInFilter
 {
    protected   ImagePlus   image;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        //      Verification of binary image. Taken from Landini
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
        ImageStatistics stats=imp.getStatistics();
        if (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount)
        {
                IJ.error("8-bit binary image (0 and 255) required.");
                return DONE;
        }
        image=imp;
        //----------------------------------------------------------------------
        //Ask if you want to apply the method to the stack or just to the image
        return IJ.setupDialog(imp,DOES_8G);
    }

    public void run(ImageProcessor ip)
    {
        int	height = ip.getHeight();
        int	width = ip.getWidth();
        int     x,y;
	int[][]	pixels=new int [width+2][height+2];

        //----------------------------------------------------------------------
        //A copy of the original image is done
        //----------------------------------------------------------------------
 	for(y=0;y<height;y++)
            for(x=0;x<width;x++)
                pixels[x+1][y+1]=ip.getPixel(x,y);//original
        //----------------------------------------------------------------------
        //The majority black filter is applied
	for (y=1;y<=height;y++)
            for(x=1;x<=width;x++)
                if(pixels[x][y]>0&(pixels[x-1][y-1]+pixels[x][y-1]+pixels[x+1][y-1]+pixels[x-1][y]+pixels[x+1][y]+pixels[x-1][y+1]+pixels[x][y+1]+pixels[x+1][y+1])<=510)
                    ip.putPixel(x-1,y-1,0);
        //----------------------------------------------------------------------
    image.show();
    image.updateAndDraw();
    }

    void showAbout()
    {
        IJ.showMessage("About MajorityBLACK_...",
        "This plug-in filter eliminates white pixels surrounded by more than a quantity of black pixels.");
    }
}
