/**
 * Manual_region_growing_3D6C.java
 * Created on 2th November 2007, 18:23
 *
 * Non recursive region growing algorithm by using queues. The seed of each
 * region growing is given by the user by clicking in it. 
 * This plugin do not create a new image
 *
 * Copyright (c)2007 by Manuel Guillermo Forero-Vargas
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import ij.*;
import ij.gui.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import java.awt.event.*;//Employed to accept events

public class Manual_region_growing_3D6C implements PlugInFilter,MouseListener,KeyListener
{
    protected   ImagePlus   imp;
    static int  minGrayLevel=20,threshold=10;
    boolean recibinglabels=true;
    ImageCanvas canvas;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
        this.imp=imp;
        IJ.register(Manual_region_growing_3D6C.class);
        //----------------------------------------------------------------------
        //Creates a dialog box,allowing the user to enter the region growing threshold
        GenericDialog gd=new GenericDialog("Region growing");
        gd.addNumericField("Threshold:",threshold,0);
        gd.addNumericField("Minimum gray level:",minGrayLevel,0);
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        threshold=(int)gd.getNextNumber();
        minGrayLevel=(int)gd.getNextNumber();
        //----------------------------------------------------------------------
        //Ask if you want to apply the method to the stack or just to the image
        return DOES_8G+DOES_8C+STACK_REQUIRED+NO_UNDO;
    }

    public void run(ImageProcessor ip)
    {
        //----------------------------------------------------------------------
        //Routines to abilitate the mouse
        canvas=imp.getWindow().getCanvas();
        canvas.addMouseListener(this);
        canvas.addKeyListener(this);
        //----------------------------------------------------------------------
        IJ.showMessage("Click once on each object.\n Then,Press ENTER to end");
        while (recibinglabels)//Receive the points clicked by the user
        {
        }
        canvas.removeMouseListener(this);
        canvas.removeKeyListener(this);
        imp.unlock();
    }

    public void keyPressed(KeyEvent e)
    {
        if(e.getKeyCode()==KeyEvent.VK_ENTER)
            recibinglabels=false;
    }

    public void keyReleased(KeyEvent e){}
    public void keyTyped(KeyEvent e){}

    public void mouseClicked(MouseEvent e)
    {
        double mean;//Average of the pixels' intensities belonging to the region
        int a;
        int pixelsNumber;//Number of pixels of the region
        int sum;//Sum of the pixels' intensities belonging to the region
        ImageProcessor ip=imp.getProcessor();
        //----------------------------------------------------------------------
        // 0. Inicializar los datos
        //----------------------------------------------------------------------
        final int height=ip.getHeight();
        final int width=ip.getWidth();
        final int depth=imp.getStackSize();
        final int depthminus1=depth-1;
        //----------------------------------------------------------------------
        int x,y,z;
        byte[][][] pixels=new byte[depth][width][height];
        byte[][][] result=new byte[depth][width][height];
        byte[] pixelsO;
        ImageStack stackIm=imp.getStack();
        //----------------------------------------------------------------------
        //Does a copy of the original stack 
        for (z=0;z<depth;z++)
        {
            pixelsO=(byte[])stackIm.getProcessor(z+1).getPixels();
            for(y=0;y<height;y++)
            {
                a=y*width;
                for(x=0;x<width;x++)
                    pixels[z][x][y]=pixelsO[a+x];
            }
        }
        //------------------------------------------------------------------
        Queue queue=new Queue(256);//256 grey levels
        z=imp.getCurrentSlice()-1;
        x=canvas.offScreenX(e.getX());//Take into account if the image is zoomed
        y=canvas.offScreenY(e.getY());
        if ((pixels[z][x][y]&0xff)>minGrayLevel)
        {
            result[z][x][y]=(byte)255;
            queue.push(new Pointv3D(x,y,z),pixels[z][x][y]&0xff);
            sum=pixels[z][x][y]&0xff;//Just one seed by region
            mean=(double)sum;
            pixelsNumber=1;
            queue.updatePriority();
            //------------------------------------------------------------------
            int i,iMax,q,X=0,Y=0,Z=0;
            Pointv3D p;
            while(queue.notEmpty())
            {
                IJ.showStatus("Manual region growing...");
                p=((Pointv3D)queue.pop());
                x=p.x;//x ordinate
                y=p.y;//y ordinate
                z=p.z;//z ordinate
                q=pixels[z][x][y]&0xff;
                //--------------------------------------------------------------
                //Revision of neighbours
                i=z==0?1:0;//Region growing in the first slice
                iMax=(z==depthminus1)?5:6;//Region growing in the last slice
                for (;i<iMax;i++)
                {
                    switch (i)
                    {
                        case 0://(z-1,x,y)
                            X=x;
                            Y=y;
                            Z=z-1;
                            break;
                        case 1://(z,x+1,y)
                            X=x+1;
                            Y=y;
                            Z=z;
                            break;
                        case 2://(z,x,y-1)
                            X--;
                            Y--;
                            break;
                        case 3://(z,x-1,y)
                            X--;
                            Y++;
                            break;
                        case 4://(z,x,y+1)
                            X++;
                            Y++;
                            break;
                        case 5://(z+1,x,y)
                            Y--;
                            Z++;
                            break;
                    }
                    if(X>=0&&X<width&&Y>=0&&Y<height&&result[Z][X][Y]==0&&(pixels[Z][X][Y]&0xff)>minGrayLevel&&(Math.abs((pixels[Z][X][Y]&0xff)-mean)<=threshold))
                    {
                        queue.push(new Pointv3D(X,Y,Z),q);
                        result[Z][X][Y]=(byte)255;
                        sum+=pixels[Z][X][Y]&0xff;
                        mean=(double)sum/++pixelsNumber;
                    }
                }
            }
            //----------------------------------------------------------------------
            //Charge the result
            for (z=0;z<depth;z++)
            {
                pixelsO=(byte[])stackIm.getProcessor(z+1).getPixels();
                for(y=0;y<height;y++)
                {
                    a=y*width;
                    for(x=0;x<width;x++)
                        if (result[z][x][y]==(byte)255)
                            pixelsO[a+x]=(byte)255;
                }
            }
            imp.show();
            imp.updateAndDraw();
        }
    }

    public void mouseDragged(MouseEvent e){}
    public void mouseEntered(MouseEvent e){}
    public void mouseExited(MouseEvent e){}
    public void mouseMoved(MouseEvent e){}
    public void mousePressed(MouseEvent e){}
    public void mouseReleased(MouseEvent e){}

    void showAbout()
    {
        IJ.showMessage("About Manual_region_growing_3D6C...",
        "This plug-in applies an iterative manual region growing algorithm to a stack of 8-bit images.");
    }
}
