/**
 * Outlier_thresholding.java
 * Created on 18 July 2007, 14:13
 *
 * Outlier threshold algorithm
 * This plugin do not create a new image.
 *
 * This method suposes the image has just one main mode.
 * A Gaussian function is aproximated to the shape of the mode by minimum error.
 * The threshold is found as a outlier higher or lower than the mean of the
 * mode. 
 * t=mean+- a * standarddeviation.
 * a=3 by default.
 *
 * Copyright (c) 2007 by Manuel Guillermo Forero-Vargas
 * e-mail: mgforero@yahoo.es
 *
 * This plugin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this plugin; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import java.awt.*;
import ij.gui.*;

public class Outlier_thresholding implements PlugInFilter
{
    static boolean lowOutlier=false;//Select the higher outlier
    static double a=3;//Number of standard deviations from the meaan
    private double mean=0;
    private double sd;
    private double mult;
    private int minthreshold=0;
//    int number=0;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        //----------------------------------------------------------------------
        //Creates a dialog box, allowing the user to enter the requested lower
        //or higher outlier and the number of standard deviations.
        //----------------------------------------------------------------------
        GenericDialog gd=new GenericDialog("Outlier threshold");
        gd.addCheckbox("lower outlier",lowOutlier);
        gd.addNumericField("Standard deviations",a,1,3,"pixels");
        gd.addNumericField("Minimum threshold",minthreshold,0,3,"");
        gd.showDialog();
        if (gd.wasCanceled())
            return DONE;
        lowOutlier=(boolean)gd.getNextBoolean();
        a=(int)gd.getNextNumber();
        minthreshold=(int)gd.getNextNumber();
        return IJ.setupDialog(imp,DOES_8G+DOES_8C);
    }

    public void run(ImageProcessor ip)
    {
        double error=0;
	double errorMin=99999999E200;
        double mean_t=0;
        double sd_t=0;
        double mult_t=0;
        int i,q,sum;
	int threshold=0;
	int minq,maxq;//Limits values of the histogram
        int[] histogram=((ByteProcessor) ip).getHistogram();

        //--------------------------------------------------------------
        //Gets limits of the histogram
        //Returns the first intensity level that appears in the histogram
        minq=0;
        while (histogram[minq]==0)//Sweeps the histogram
            minq++;//Returns the first value!=0
	//Returns the last intensity level that appears in the histogram
        maxq=255;//Levels-1;
        while (histogram[maxq]==0)
            maxq--;
        //----------------------------------------------------------------------
        //Evaluation of the pdf of the first mode
        for (q=minq;q<maxq;q++)
        {
            mean=0;
            sd=0;
            for(sum=0,i=q;i<=maxq;i++)
            {
                sum+=histogram[i];
                mean+=i*histogram[i];
            }
            mean/=sum;
            for(i=q;i<=maxq;i++) 
                sd+=histogram[i]*(i-mean)*(i-mean);
            sd/=sum;
            mult=histogram[(int)(mean+0.5)];
            //------------------------------------------------------------------
            //Evaluation of the error
            for(error=0,i=minq;i<=maxq;i++)
        	error+=(gamma2(i)-histogram[i])*(gamma2(i)-histogram[i]);
//            error/=256;
            //------------------------------------------------------------------
            if(error<errorMin)
            {
		errorMin=error;
		threshold=q;
                mean_t=mean;
                sd_t=sd;
                mult_t=mult;//Required to show the final curve ir runHistogram is true
	    }
	}
        mean=mean_t;
        sd=sd_t;
        mult=mult_t;
        threshold=lowOutlier?(int)(mean-a*Math.sqrt(sd)):(int)(mean+a*Math.sqrt(sd));
        if (threshold<minthreshold)
            threshold=minthreshold;
        ip.threshold(threshold);
    }

    public final double gamma2(int i)
    {
	return (sd==0)?0:mult*Math.exp(-(i-mean)*(i-mean)/(2*sd));
    }

    void showAbout()
    {
        IJ.showMessage("About Outlier_thresholding...",
        "This plug-in filter calculates the outlier threshold of 8-bit images.");
    }
}
