/**
* Recursive_Weighted_Distance_3D.java
* Created on 12th August 2007, 17:09
* e-mail: mgforero@yahoo.es
*
* Function: This plugin calculates the 3D distance transform based on the
* algorithm described by Borgefors, instead of the plugin DT3D_ of Christian Henden.
* This plugin does not create a new image.
*
* Copyright (c) 2007 by Manuel Guillermo Forero-Vargas
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;

public class Recursive_Weighted_Distance_3D implements PlugInFilter
{
    private ImagePlus imp;

    public int setup(String arg,ImagePlus imp)
    {
        this.imp=imp;
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        return DOES_8G+DOES_8C+STACK_REQUIRED+NO_UNDO;
    }

    public void run(ImageProcessor ip)
    {
        ImageStack  stack=imp.getStack();
        byte[] pixelsBin;
        final int w=ip.getWidth();
        final int h=ip.getHeight();
        final int d=imp.getStackSize();
        int a,b,c,e,i,min=99999,x,xx,y,yy,z,zz;
        final int[] f=new int[]{5,4,5,4,3,4,5,4,5,4,3,4,3};
        int[][]	pixelsTE=new int[d][w*h];

        //----------------------------------------------------------------------
        // Forward iteration
        for (z=0;z<d;z++)
        {
            pixelsBin=(byte[])stack.getPixels(z+1);
            for (y=0;y<h;y++)
            {
                a=y*w;
                for (x=0;x<w;x++)
                    if (pixelsBin[a+x]!=0)//Background=0
                    {
                        for (i=0,min=99999,zz=-1;zz<1;zz++)
                        {
                            if ((b=z+zz)<0)
                            {
                                min=3;//It is assumed that outside the image the ROIs do not exist.
                                break;
                            }
                            else
                                for (yy=-1;yy<2;yy++)
                                {
                                    c=y+yy;
                                    if (c<0||c==h)
                                    {
                                        min=3;
                                        zz=2;
                                        break;
                                    }
                                    else
                                    {
                                        c*=w;
                                        for (xx=-1;xx<2;xx++)
                                        {
                                            if (x+xx<0||x+xx==w)
                                            {
                                                min=3;
                                                yy=zz=2;
                                                break;
                                            }
                                            else
                                            {
                                                if ((e=f[i]+pixelsTE[b][c+x+xx])<min)
                                                    min=e;
                                                i++;
                                                if (zz==0&&yy==0&&xx==-1)
                                                {
                                                    zz=yy=2;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                }
                        }
                        pixelsTE[z][a+x]=min;
                    }
            }
        }
        // Backward iteration
        for (z=d-1;z>=0;z--)
        {
            pixelsBin=(byte[])stack.getPixels(z+1);
            for (y=h-1;y>=0;y--)
            {
                a=y*w;
                for (x=w-1;x>=0;x--)
                    if (pixelsBin[a+x]!=0)//Background=0
                    {
                        for (i=0,min=99999,zz=1;zz>-1;zz--)
                        {
                            if ((b=z+zz)==d)
                            {
                                min=3;//It is assumed that outside the image the ROIs do not exist.
                                break;
                            }
                            else
                                for (yy=1;yy>-2;yy--)
                                {
                                    c=y+yy;
                                    if (c<0||c==h)
                                    {
                                        min=3;
                                        zz=-2;
                                        break;
                                    }
                                    else
                                    {
                                        c*=w;
                                        for (xx=1;xx>-2;xx--)
                                        {
                                            if (x+xx<0||x+xx==w)
                                            {
                                                min=3;
                                                yy=zz=-2;
                                                break;
                                            }
                                            else
                                            {
                                                if ((e=f[i]+pixelsTE[b][c+x+xx])<min)
                                                    min=e;
                                                i++;
                                                if (zz==0&&yy==0&&xx==1)
                                                {
                                                    zz=yy=-2;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                }
                        }
                        e=a+x;
                        if (min<pixelsTE[z][e])
                            pixelsTE[z][e]=min;
                        pixelsBin[e]=(byte)pixelsTE[z][e];
                    }
            }
        }
        imp.show();
        imp.updateAndDraw();
    }

    void showAbout()
    {
        IJ.showMessage("About Recursive_Weighted_Distance_3D...",
        "This plug-in gets the 3D weighted distances of a binary image.");
    }
}
