/**
* Volume_Measure.java
* Created on 13th September 2009, 12:35 by Manuel Guillermo Forero-Vargas
* e-mail: m.g.forerovargas@bham.ac.uk, mgforero@yahoo.es
*
* Volume measure algorithm
* Function: This plug-in measures the whole volume of the objects on the stack.
* This plugin receive a binarise stacks
* 
* This plugin do not create a new stack.
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import ij.*;
import ij.process.*;
import ij.plugin.filter.*;
import ij.measure.Calibration;

public class Volume_Measure implements PlugInFilter
{
	protected  ImagePlus imp;
    protected   ImageStack stack;
    protected   StackStatistics istats;

    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
    	{
            showAbout();
            return DONE;
        }
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
         //----------------------------------------------------------------------
        ImageStatistics stats=imp.getStatistics();
        if (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount)
        {
            IJ.error("8-bit binary imp (0 and 255) required.");
            return DONE;
        }
	this.imp=imp;
        //----------------------------------------------------------------------
        StackStatistics sstats=new StackStatistics (imp);
        istats=new StackStatistics(imp,(int)sstats.histMax+1,0,sstats.histMax);//Recalculates the histogram using integer beans
        return DOES_8G+DOES_8C+STACK_REQUIRED;
    }

    public void run(ImageProcessor ip)
    {
        Calibration cal=imp.getCalibration();
        double vol=istats.histogram[255]*cal.pixelWidth*cal.pixelHeight*cal.pixelDepth;
        IJ.log("Volume: "+vol+" cubic units");
    }

    void showAbout()
    {
        IJ.showMessage("About Volume_Measure...",
        "This plug-in filter measures the total volume of the 3D-objects in the stack.");
    }
}
